package com.dragon.car.utils;

import com.dragon.common.core.constant.SecurityConstants;
import com.dragon.common.core.domain.Result;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.*;
import org.springframework.stereotype.Component;
import org.springframework.web.context.request.RequestContextHolder;
import org.springframework.web.context.request.ServletRequestAttributes;

import javax.servlet.http.HttpServletRequest;
import java.util.Arrays;
import java.util.Enumeration;
import java.util.Optional;
import java.util.UUID;

/**
 * @author : Administrator
 * @description : 更新AOP日志
 */
@Component
@Aspect
@Slf4j
public class LogAOP {
	/**
	 * com.dragon.car.service.controller：表示匹配这个包下的类。
	 * ..：表示匹配任意子包。
	 * ：表示匹配任意类。
	 * .*：表示匹配任意方法。
	 * (..)：表示匹配任意参数列表的方法。
	 */
	private static final String POINT = "execution(* com.dragon.car.service.controller..*.*(..))";
	private static final ThreadLocal<Long> START_TIME = ThreadLocal.withInitial (System::currentTimeMillis);
	private static final ThreadLocal<String> SEQUENCE = ThreadLocal.withInitial (
			() -> UUID
					      .randomUUID ()
					      .toString ()
					      .replaceAll ("-", ""));
	
	@Pointcut (POINT)
	public void executeService () {
	
	}
	
	/** 前置通知 */
	@Before ("executeService()")
	public void before (JoinPoint joinPoint) {
		ServletRequestAttributes attributes = (ServletRequestAttributes) RequestContextHolder.getRequestAttributes ();
		HttpServletRequest request = Optional.ofNullable (attributes).map (ServletRequestAttributes::getRequest).orElse (null);
		String ipAddress = Optional.ofNullable (request).map (r -> r.getHeader ("X-Forwarded-For")).orElse (null);
		
		if (ipAddress == null || ipAddress.isEmpty () || "unknown".equalsIgnoreCase (ipAddress)) {
			Optional.ofNullable (request).map (HttpServletRequest::getRemoteAddr).orElse (null);
		}
		else {
			ipAddress = ipAddress.split (",")[0];
		}
		String headers = Optional.ofNullable (request).map (HttpServletRequest::getHeaderNames)
		                         .map (Enumeration::asIterator)
		                         .stream ()
		                         .map (name -> name + ":" + request.getHeader (name.toString ()))
		                         .reduce ("", (a, b) -> a + "|" + b);
		
		
		String parameters = Optional.ofNullable (request).map (HttpServletRequest::getParameterMap)
		                            .map (map -> map.entrySet ()
		                                            .stream ()
		                                            .map (entry -> entry.getKey () + ":" + Arrays.toString (entry.getValue ()))
		                                            .reduce ("", (a, b) -> a + "|" + b))
		                            .orElse ("");
		log.info (
				"""
						trace log begin --> sequence::{},
						userName::{},
						RealIp::{},
						User- Agent::{},
						requestUri::{},
						header::{},
						requestMethod::{},
						request::{},
						args::{}
						""",
				
				SEQUENCE.get (), SecurityConstants.DETAILS_USERNAME, ipAddress,
				Optional.ofNullable (request)
				        .map (r -> r.getHeader ("User-Agent"))
				        .orElse (""),
				Optional.ofNullable (request)
				        .map (HttpServletRequest::getRequestURI)
				        .orElse (""),
				headers,
				Optional.ofNullable (request)
				        .map (HttpServletRequest::getMethod)
				        .orElse (""),
				Optional.ofNullable (request)
				        .map (HttpServletRequest::toString)
				        .orElse (""),
				Arrays.deepToString (joinPoint.getArgs ())
		         );
		
		START_TIME.set (System.currentTimeMillis ());
		
	}
	
	/** 后置 */
	@AfterReturning (pointcut = "executeService()", returning = "result")
	public void after (Object result) {
		String responseBody = result instanceof Result ? result.toString () : "not standard response";
		log.info ("""
				          trace log end -->sequence::{}, used::{}ms, responseBody::{}
				          """, SEQUENCE.get (), (System.currentTimeMillis () - START_TIME.get ()), responseBody);
		SEQUENCE.remove ();
		START_TIME.remove ();
	}
	
	/** 异常 */
	@AfterThrowing (pointcut = "executeService()", throwing = "e")
	public void afterThrowing (JoinPoint joinPoint, Exception e) {
		log.error (
				"""
						trace log exception --> sequence::{},
						message::{},
						requestMethod::{},
						requestUri::{},
						args::{}
						""",
				SEQUENCE.get (),
				e.getMessage (),
				Optional.ofNullable (RequestContextHolder.getRequestAttributes ())
				        .map (attributes -> ((ServletRequestAttributes) attributes).getRequest ().getMethod ())
				        .orElse (""),
				Optional.ofNullable (RequestContextHolder.getRequestAttributes ())
				        .map (attributes -> ((ServletRequestAttributes) attributes).getRequest ().getRequestURI ())
				        .orElse (""),
				Arrays.deepToString (joinPoint.getArgs ()),
				e
		          );
	}
	
}
