package com.dragon.car.service.service.impl;

import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.dragon.car.domain.FaultCode;
import com.dragon.car.domain.req.ReqFault;
import com.dragon.car.domain.resp.RespFaultCode;
import com.dragon.car.service.mapper.FaultMapper;
import com.dragon.car.service.service.FaultService;
import com.dragon.common.core.domain.Result;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Objects;
import java.util.Optional;

/**
 * @author zhn
 * @description 故障管理业务层
 * <p>
 * 判断 reqFault 操作的 operation 属于啥 c添加 r查询 u修改 d删除
 * <p>
 * 判断 reqFault 操作状态的 operationStatus 对状态码是否启用 0/1
 * <p>
 * 判断 reqFault 操作键的 operation keu 状态码
 * <p>
 * 判断 reqFault 操作值的 operation value 自己看的值
 **/
@Service
public class FaultServiceImpl implements FaultService {
	@Autowired
	FaultMapper faultMapper;
	
	/**
	 * 故障管理-查询
	 *
	 * @return RespFaultCode 故障返回实体类
	 */
	@Override
	public Result<List<RespFaultCode>> queryFaultCode () {
		return Result.success (faultMapper
				                       .selectList (Wrappers.query ())
				                       .stream ()
				                       .map (a -> new RespFaultCode (a.getId (), a.getCode (), a.getName (), a.getStatus ()))  // 使用map方法将A对象转换为B对象
				                       .toList ());
	}
	
	/**
	 * 故障管理-新增
	 *
	 * @param reqFault 故障实体类
	 *
	 * @return 返回 0/1
	 */
	@Override
	public Result insertFaultCode (ReqFault reqFault) {
		return Optional.of (reqFault)
		               .filter (fault -> "c".equals (fault.getOperation ()))
		               .map (fault -> new FaultCode (fault.getOperationKey (), fault.getOperationValue (), fault.getOperationStatus ()))
		               .map (code -> faultMapper.insert (code) > 0 ? Result.success ("insert 成功") : Result.error ("insert 失败"))
		               .orElse (Result.error ("添加操作异常"));
	}
	
	/**
	 * 故障管理-修改
	 *
	 * @param reqFault 故障实体类
	 *
	 * @return 返回 0/1
	 */
	@Override
	public Result updateFaultCode (ReqFault reqFault) {
		// 1. 判断操作状态是 "u"，传输的 key 和 value 非空，状态非 null 非 ""
		if ("u".equals (reqFault.getOperation ())) {
			return Optional.of (reqFault)
			               .filter (fault -> fault.getOperationKey () != null && !fault.getOperationKey ().isEmpty ())
			               .filter (fault -> fault.getOperationValue () != null && !fault.getOperationValue ().isEmpty ())
			               .map (fault -> Result.success ("update成功"))
			               .orElse (Result.error ("操作状态为 'u'，但 key 或 value 为空"));
		}
		return Result.error ("操作状态不为 'u'");
	}
	
	
	/**
	 * 故障管理-删除
	 *
	 * @param reqFault 故障实体类
	 *
	 * @return 返回 0/1
	 */
	
	@Override
	public Result deleteFaultCode (ReqFault reqFault) {
		String operation = reqFault.getOperation ();
		String code = reqFault.getOperationKey ();
		String operationStatus = reqFault.getOperationStatus ();
		
		return switch (operation) {
			case "d" -> {
				if (Objects.equals (operationStatus, "0")) {
					int deleteCount = faultMapper.delete (Wrappers.lambdaQuery (FaultCode.class).eq (FaultCode::getCode, code));
					yield deleteCount > 0 ? Result.success ("彻底删除成功") : Result.error ("彻底删除失败");
				}
				else if (Objects.equals (operationStatus, "1")) {
					int updateCount = faultMapper.update (new FaultCode (code, null, "0"), Wrappers.lambdaQuery (FaultCode.class).eq (FaultCode::getCode, code));
					yield updateCount > 0 ? Result.success ("逻辑删除成功") : Result.error ("逻辑删除失败");
				}
				else {
					yield Result.error ("操作状态不合法");
				}
			}
			default -> Result.error ("操作状态不为 'd'");
		};
	}
}
